@@ -9,6 +9,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
+#include <sys/mman.h>
 #include <unistd.h>
 
 #include "config.h"
@@ -64,13 +65,16 @@ const struct argp_option options[] = {
                            "fake success for some syscalls with seccomp(2)"},
 #endif
    { "set-env",        -6, "ARG",  OPTION_ARG_OPTIONAL,
-                           "set environment variables per ARG"},
+                           "set env. variables per ARG (newline-delimited)"},
+   { "set-env0",      -15, "ARG",  OPTION_ARG_OPTIONAL,
+                           "set env. variables per ARG (null-delimited)"},
    { "storage",       's', "DIR",  0, "set DIR as storage directory"},
    { "uid",           'u', "UID",  0, "run as UID within container" },
    { "unsafe",        -13, 0,      0, "do unsafe things (internal use only)" },
    { "unset-env",      -7, "GLOB", 0, "unset environment variable(s)" },
    { "verbose",       'v', 0,      0, "be more verbose (can be repeated)" },
    { "version",       'V', 0,      0, "print version and exit" },
+   { "warnings",      -16, "NUM",  0, "log NUM warnings and exit" },
    { "write",         'w', 0,      0, "mount image read-write"},
    { 0 }
 };
@@ -99,14 +103,17 @@ int join_ct(int cli_ct);
 char *join_tag(char *cli_tag);
 int parse_int(char *s, bool extra_ok, char *error_tag);
 static error_t parse_opt(int key, char *arg, struct argp_state *state);
+void parse_set_env(struct args *args, char *arg, int delim);
 void privs_verify_invoking();
 char *storage_default(void);
+extern void warnings_reprint(void);
 
 
 /** Global variables **/
 
 const struct argp argp = { options, parse_opt, args_doc, usage };
 extern char **environ;  // see environ(7)
+extern char *warnings;
 
 
 /** Main **/
@@ -118,8 +125,15 @@ int main(int argc, char *argv[])
    int arg_next;
    char ** c_argv;
 
+   // initialize “warnings” buffer
+   warnings = mmap(NULL, WARNINGS_SIZE, PROT_READ | PROT_WRITE,
+                   MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+   T_ (warnings != MAP_FAILED);
+
    privs_verify_invoking();
 
+   Z_ (atexit(warnings_reprint));
+
 #ifdef ENABLE_SYSLOG
    syslog(LOG_USER|LOG_INFO, "uid=%u args=%d: %s", getuid(), argc,
           argv_to_string(argv));
@@ -266,12 +280,12 @@ void fix_environment(struct args *args)
          Te (false, "unreachable code reached");
          break;
       case ENV_SET_DEFAULT:
-         ed.arg.vars = env_file_read("/ch/environment");
+         ed.arg.vars = env_file_read("/ch/environment", ed.arg.delim);
          // fall through
       case ENV_SET_VARS:
          for (size_t j = 0; ed.arg.vars[j].name != NULL; j++)
             env_set(ed.arg.vars[j].name, ed.arg.vars[j].value,
-                    args->c.env_expand );
+                    args->c.env_expand);
          break;
       case ENV_UNSET_GLOB:
          env_unset(ed.arg.glob);
@@ -381,7 +395,6 @@ int parse_int(char *s, bool extra_ok, char *error_tag)
 static error_t parse_opt(int key, char *arg, struct argp_state *state)
 {
    struct args *args = state->input;
-   struct env_delta ed;
    int i;
 
    switch (key) {
@@ -397,25 +410,15 @@ static error_t parse_opt(int key, char *arg, struct argp_state *state)
       args->c.join_pid = parse_int(arg, false, "--join-pid");
       break;
    case -6: // --set-env
-      if (arg == NULL)
-         ed.action = ENV_SET_DEFAULT;
-      else {
-         ed.action = ENV_SET_VARS;
-         if (strchr(arg, '=') == NULL)
-            ed.arg.vars = env_file_read(arg);
-         else {
-            ed.arg.vars = list_new(sizeof(struct env_var), 1);
-            ed.arg.vars[0] = env_var_parse(arg, NULL, 0);
-         }
-      }
-      list_append((void **)&(args->env_deltas), &ed, sizeof(ed));
+      parse_set_env(args, arg, '\n');
       break;
-   case -7: // --unset-env
-      Te (strlen(arg) > 0, "--unset-env: GLOB must have non-zero length");
-      ed.action = ENV_UNSET_GLOB;
-      ed.arg.glob = arg;
-      list_append((void **)&(args->env_deltas), &ed, sizeof(ed));
-      break;;
+   case -7: { // --unset-env
+        struct env_delta ed;
+        Te (strlen(arg) > 0, "--unset-env: GLOB must have non-zero length");
+        ed.action = ENV_UNSET_GLOB;
+        ed.arg.glob = arg;
+        list_append((void **)&(args->env_deltas), &ed, sizeof(ed));
+      } break;
    case -9: // --no-passwd
       args->c.private_passwd = true;
       break;
@@ -456,6 +459,14 @@ static error_t parse_opt(int key, char *arg, struct argp_state *state)
       args->seccomp_p = true;
       break;
 #endif
+   case -16: // --warnings
+      for (int i = 1; i <= parse_int(arg, false, "--warnings"); i++)
+         WARNING("this is warning %d!", i);
+      exit(0);
+      break;
+   case -15: // --set-env0
+      parse_set_env(args, arg, '\0');
+      break;
    case 'b': {  // --bind
          char *src, *dst;
          for (i = 0; args->c.binds[i].src != NULL; i++) // count existing binds
@@ -529,6 +540,26 @@ static error_t parse_opt(int key, char *arg, struct argp_state *state)
    return 0;
 }
 
+void parse_set_env(struct args *args, char *arg, int delim)
+{
+   struct env_delta ed;
+
+   if (arg == NULL) {
+      ed.action = ENV_SET_DEFAULT;
+      ed.arg.delim = delim;
+   } else {
+      ed.action = ENV_SET_VARS;
+      if (strchr(arg, '=') == NULL)
+         ed.arg.vars = env_file_read(arg, delim);
+      else {
+         ed.arg.vars = list_new(sizeof(struct env_var), 1);
+         ed.arg.vars[0] = env_var_parse(arg, NULL, 0);
+      }
+   }
+   list_append((void **)&(args->env_deltas), &ed, sizeof(ed));
+}
+
+
 /* Validate that the UIDs and GIDs are appropriate for program start, and
    abort if not.
 
