@@ -16,7 +16,6 @@ import build_cache as bu
 import filesystem as fs
 import force
 import image as im
-import pull
 
 
 ## Globals ##
@@ -75,7 +74,7 @@ def main(cli_):
       if (base == "Dockerfile"):
          cli.tag = ext_all
          ch.VERBOSE("inferring name from Dockerfile extension: %s" % cli.tag)
-      elif (ext_last == "dockerfile"):
+      elif (ext_last in ("df", "dockerfile")):
          cli.tag = base_all
          ch.VERBOSE("inferring name from Dockerfile basename: %s" % cli.tag)
       elif (os.path.abspath(cli.context) != "/"):
@@ -89,12 +88,12 @@ def main(cli_):
       ch.INFO("inferred image name: %s" % cli.tag)
 
    # --force and friends.
-   if (cli.force_cmd and cli.force == "fakeroot"):
+   if (cli.force_cmd and cli.force == ch.Force_Mode.FAKEROOT):
       ch.FATAL("--force-cmd and --force=fakeroot are incompatible")
    if (not cli.force_cmd):
       cli.force_cmd = force.FORCE_CMD_DEFAULT
    else:
-      cli.force = "seccomp"
+      cli.force = ch.Force_Mode.SECCOMP
       # convert cli.force_cmd to parsed dict
       force_cmd = dict()
       for line in cli.force_cmd:
@@ -102,10 +101,10 @@ def main(cli_):
          force_cmd[cmd] = args
       cli.force_cmd = force_cmd
    ch.VERBOSE("force mode: %s" % cli.force)
-   if (cli.force == "seccomp"):
+   if (cli.force == ch.Force_Mode.SECCOMP):
       for (cmd, args) in cli.force_cmd.items():
          ch.VERBOSE("force command: %s" % ch.argv_to_string([cmd] + args))
-   if (    cli.force == "seccomp"
+   if (    cli.force == ch.Force_Mode.SECCOMP
        and ch.cmd([ch.CH_BIN + "/ch-run", "--feature=seccomp"],
                   fail_ok=True) != 0):
       ch.FATAL("ch-run was not built with seccomp(2) support")
@@ -200,9 +199,9 @@ def main(cli_):
    if (ml.instruction_total_ct == 0):
       ch.FATAL("no instructions found: %s" % cli.file)
    assert (ml.inst_prev.image_i + 1 == image_ct)  # should’ve errored already
-   if (cli.force and ml.miss_ct != 0):
+   if ((cli.force != ch.Force_Mode.NONE) and ml.miss_ct != 0):
       ch.INFO("--force=%s: modified %d RUN instructions"
-              % (cli.force, forcer.run_modified_ct))
+              % (cli.force.value, forcer.run_modified_ct))
    ch.INFO("grown in %d instructions: %s"
            % (ml.instruction_total_ct, ml.inst_prev.image))
    # FIXME: remove when we’re done encouraging people to use the build cache.
@@ -1156,9 +1155,9 @@ class Run(Instruction):
    def str_name(self):
       # Can’t get this from the forcer object because it might not have been
       # initialized yet.
-      if (cli.force is None):
-         tag = ""
-      elif (cli.force == "fakeroot"):
+      if (cli.force == ch.Force_Mode.NONE):
+         tag = ".N"
+      elif (cli.force == ch.Force_Mode.FAKEROOT):
          # FIXME: This causes spurious misses because it adds the force tag to
          # *all* RUN instructions, not just those that actually were modified
          # (i.e, any RUN instruction will miss the equivalent RUN without
@@ -1166,7 +1165,7 @@ class Run(Instruction):
          # modifications until the result is checked out, which happens after
          # we check the cache. See issue #1339.
          tag = ".F"
-      elif (cli.force == "seccomp"):
+      elif (cli.force == ch.Force_Mode.SECCOMP):
          tag = ".S"
       else:
          assert False, "unreachable code reached"
