@@ -27,7 +27,6 @@
 
 import configparser
 import datetime
-import enum
 import glob
 import hashlib
 import itertools
@@ -38,7 +37,6 @@ import shutil
 import stat
 import tempfile
 import textwrap
-import time
 
 import charliecloud as ch
 import image as im
@@ -158,7 +156,7 @@ def have_deps(required=True):
    global git
    git = shutil.which("git")
    if (git is None):
-      (FATAL if required else VERBOSE)("no git(1) found")
+      (ch.FATAL if required else ch.VERBOSE)("no git(1) found")
       return False
    # As of 2.34.1, we get: "git version 2.34.1\n".
    return ch.version_check([git, "--version"], GIT_MIN, required=required)
@@ -243,6 +241,13 @@ class File_Metadata:
    #   hardlink_to ... If non-None, file is a hard link to this other file,
    #                   stored as a Path object relative to the image root.
    #
+   #   xattrs ........ Extended attributes of the file, stored as a dictionary.
+   #                   The keys take the form “namespace.name”, where
+   #                   “namespace” is the namespace the xattr belongs to (e.g.
+   #                   “user” or “system”) and “name” is the actual name of the
+   #                   xattr. The value in the dictionary is the value assigned
+   #                   to the xattr.
+   #
    # Attributes not stored (recomputed on unpickle):
    #
    #   image_root .... Absolute path to the image directory to which path is
@@ -269,6 +274,10 @@ class File_Metadata:
       self.dont_restore = False
       self.hardlink_to = None
       self.large_name = None
+      self.xattrs = dict()
+      if ch.save_xattrs:
+         for xattr in os.listxattr(self.path_abs, follow_symlinks=False):
+            self.xattrs[xattr] = os.getxattr(self.path_abs, xattr, follow_symlinks=False)
 
    def __getstate__(self):
       return { a:v for (a,v) in self.__dict__.items()
@@ -472,6 +481,10 @@ class File_Metadata:
          ch.ossafe(os.mkfifo, "can’t make FIFO: %s" % self.path, self.path_abs)
       elif (self.path.git_incompatible_p):
          self.path_abs.git_escaped.rename_(self.path_abs)
+      if ch.save_xattrs:
+         for (xattr, val) in self.xattrs.items():
+            ch.ossafe(os.setxattr, "unable to restore xattr: %s" % xattr,
+                      self.path_abs, xattr, val, follow_symlinks=False)
       # Recurse children.
       if (len(self.children) > 0):
          for child in self.children.values():
@@ -548,11 +561,13 @@ class File_Metadata:
 
    def unpickle_fix(self, image_root, path):
       "Does no I/O."
-      # old: large_name, size: no such attribute
+      # old: large_name, size, xattrs: no such attribute
       if (not (hasattr(self, "large_name"))):
          self.large_name = None
       if (not (hasattr(self, "size"))):
          self.size = -1
+      if (not (hasattr(self, "xattrs"))):
+         self.xattrs = dict()
       # old: hardlink_to: stored as string
       if (isinstance(self.hardlink_to, str)):
          self.hardlink_to = fs.Path(self.hardlink_to)
@@ -1091,7 +1106,7 @@ class Enabled_Cache:
             # no PID file, therefore no GC running
             pass
          except OSError as x:
-            FATAL("can’t open GC PID file: %s: %s" % (pid_path, x.strerror))
+            ch.FATAL("can’t open GC PID file: %s: %s" % (pid_path, x.strerror))
          # Delete images that are worktrees referring back to the build cache.
          ch.INFO("deleting build cache")
          for d in ch.storage.unpack_base.listdir():
