@@ -1,4 +1,3 @@
-import os.path
 import re
 
 import charliecloud as ch
@@ -293,11 +292,11 @@ FORCE_CMD_DEFAULT = { "apt":     ["-o", "APT::Sandbox::User=root"],
 def new(image_path, force_mode, force_cmds):
    """Return a new forcer object appropriate for image at image_path in mode
       force_mode. If no such object can be found, exit with error."""
-   if (force_mode is None):
+   if (force_mode == ch.Force_Mode.NONE):
       return Nope()
-   elif (force_mode == "fakeroot"):
+   elif (force_mode == ch.Force_Mode.FAKEROOT):
       return Fakeroot(image_path)
-   elif (force_mode == "seccomp"):
+   elif (force_mode == ch.Force_Mode.SECCOMP):
       return Seccomp(force_cmds)
    else:
       assert False, "unreachable code reached"
